library align_positioned;

import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

////////////////////////////////////////////////////////////////////////////////////////////////////

enum Touch { inside, outside }

////////////////////////////////////////////////////////////////////////////////////////////////////

class AlignPositioned extends SingleChildRenderObjectWidget {
  //
  final Alignment alignment;
  final double dx, dy;
  final double dWidth, dHeight;
  final Touch touch;

  const AlignPositioned({
    Key key,
    this.alignment = Alignment.center,
    this.dx = 0.0,
    this.dy = 0.0,
    this.dWidth = 0.0,
    this.dHeight = 0.0,
    this.touch = Touch.inside,
    Widget child,
  })  : assert(alignment != null),
        super(key: key, child: child);

  @override
  _RenderAlignPositionedBox createRenderObject(BuildContext context) {
    return _RenderAlignPositionedBox(
      alignment: alignment,
      dx: dx ?? 0.0,
      dy: dy ?? 0.0,
      dWidth: dWidth ?? 0.0,
      dHeight: dHeight ?? 0.0,
      touch: touch,
      textDirection: Directionality.of(context),
    );
  }

  @override
  void updateRenderObject(BuildContext context, _RenderAlignPositionedBox renderObject) {
    renderObject
      ..alignment = alignment
      ..dx = dx ?? 0.0
      ..dy = dy ?? 0.0
      ..dWidth = dWidth ?? 0.0
      ..dHeight = dHeight ?? 0.0
      ..touch = touch;
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

class _RenderAlignPositionedBox extends RenderShiftedBox {
  //
  _RenderAlignPositionedBox({
    RenderBox child,
    @required double dx,
    @required double dy,
    @required double dWidth,
    @required double dHeight,
    @required Touch touch,
    Alignment alignment = Alignment.center,
    TextDirection textDirection,
  })  : assert(dx != null),
        assert(dy != null),
        assert(touch != null),
        _dx = dx,
        _dy = dy,
        _dWidth = dWidth,
        _dHeight = dHeight,
        _alignment = alignment,
        _touch = touch,
        super(child);

  // ---

  Alignment get alignment => _alignment;
  Alignment _alignment;

  set alignment(Alignment value) {
    assert(value != null);
    if (_alignment == value) return;
    _alignment = value;
    markNeedsLayout();
  }

  // ---

  Touch get touch => _touch;
  Touch _touch;

  set touch(Touch value) {
    assert(value != null);
    if (_touch == value) return;
    _touch = value;
    markNeedsLayout();
  }

  // ---

  double get dWidth => _dWidth;
  double _dWidth;

  set dWidth(double value) {
    assert(value != null);
    if (_dWidth == value) return;
    _dWidth = value;
    markNeedsLayout();
  }

  // ---

  double get dHeight => _dHeight;
  double _dHeight;

  set dHeight(double value) {
    assert(value != null);
    if (_dHeight == value) return;
    _dHeight = value;
    markNeedsLayout();
  }

  // ---

  double get dx => _dx;
  double _dx;

  set dx(double value) {
    assert(value != null);
    if (_dx == value) return;
    _dx = value;
    markNeedsLayout();
  }

  // ---

  double get dy => _dy;
  double _dy;

  set dy(double value) {
    assert(value != null);
    if (_dy == value) return;
    _dy = value;
    markNeedsLayout();
  }

  // ---

  @override
  void performLayout() {
    //
    if (child != null) {
      child.layout(constraints.loosen(), parentUsesSize: true);
      size = constraints.constrain(const Size(double.infinity, double.infinity));
      alignChild();
    } else {
      size = constraints.constrain(const Size(double.infinity, double.infinity));
    }
  }

  @protected
  void alignChild() {
    assert(child != null);
    assert(!child.debugNeedsLayout);
    assert(child.hasSize);
    assert(hasSize);
    // ---

    final BoxParentData childParentData = child.parentData;
    final Offset containerSize = _toOffset(size);
    final Offset childSize = _toOffset(child.size);

    Align
    // 1) Adds touch and alignment.
    if (_touch == Touch.inside)
      childParentData.offset = alignment.alongOffset(containerSize - childSize);
    //
    else if (_touch == Touch.outside)
      childParentData.offset = alignment.alongOffset(containerSize + childSize) - childSize;
    //
    else
      throw AssertionError();

    // 2) Adds dx and dy.
    childParentData.offset += Offset(dx, dy);

    // 3) Adds dWidth and dHeight.
    childParentData.offset += Offset(
      child.size.width * dWidth,
      child.size.height * dHeight,
    );
  }

  Offset _toOffset(Size size) => Offset(size.width, size.height);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
